/* EE 41440: Senior Design II
   Pinball 2022
   University of Notre Dame

   This code runs the 2022 Pinball Senior Design Project. It is written for an ESP32 WROOM Module running
   on a custom-designed PCB board. The microcontroller is responsible for keeping track of the score and
   game number, and reporting them on a seven-segment display, as well as determining "events" (scoring 
   and the end of the game). When events occur, the microcontroller will update the display, flash an
   LED strip, and play audio, as appropriate. In the absence of events, the microcontroller will provide
   ambience in the form of audio and an LED routine.

   Team website: http://seniordesign.ee.nd.edu/2022/Design%20Teams/Pinball/top_page.html

   Team Members:
      Alisa Nguyen
      Lauren Rymsza
      Victoria Smerdon
      Yadviga Tischenko 
      James Venditto

   This code is intended for an ESP32 WROOM Module.

   Code authors and contributors:
      Alisa Nguyen
      Lauren Rymza
      James Venditto
      
    Audio Library courtesy of Earle Philhower and Alex Wende (https://github.com/earlephilhower/ESP8266Audio)
    Multicore Code example courtesy of Rui Santos (https://randomnerdtutorials.com/esp32-dual-core-arduino-ide/)

*/

////// Global Definitions:
// Pins:
#define GAME_OVER_PIN 12                      // game over pin (beam-break sensor)
#define TARGET_PIN 16                         // target pin
#define SLINGSHOT_PIN_1 13                    // slingshot pin
#define SLINGSHOT_PIN_2 19                    // slingshot pin                        

// Logic Definitions:
#define GAME_OVER 1
#define TARGET 2
#define SLINGSHOT 3

// Global Variables:
unsigned int score = 0;                       // score variable


////// Definitions for LED:
#define BLUE 15
#define RED 4
#define GREEN 5


////// Definitions and headers for 7-Segment Display:
#include <Wire.h>
#include <Adafruit_GFX.h>
#include "Adafruit_LEDBackpack.h"

// Score Matrix:
TwoWire I2CMatrix_score = TwoWire(0);
Adafruit_7segment matrix_score = Adafruit_7segment();

// Game Number Matrix:
TwoWire I2CMatrix_game = TwoWire(1);
Adafruit_7segment matrix_game = Adafruit_7segment();


////// Definitions and headers for Audio:
#include <Arduino.h>
#include "AudioGeneratorAAC.h"
#include "AudioOutputI2S.h"
#include "AudioFileSourcePROGMEM.h"
#include "target.h"
#include "gameOver.h"
#include "slingshot.h"
#include "NDVictoryMarch.h"

// Headers for Watchdog Timer:
#include "soc/timer_group_struct.h"
#include "soc/timer_group_reg.h"

// Initialize ESP8266 Audio Library classes:
AudioFileSourcePROGMEM *file_target; 
AudioFileSourcePROGMEM *file_gameOver; 
AudioFileSourcePROGMEM *file_slingshot; 
AudioFileSourcePROGMEM *file_ambient; 
AudioGeneratorAAC *aac;
AudioOutputI2S *out;
volatile bool playing = 0;
volatile byte loadTrack = 0;      // for audio
volatile byte event = 0;          // for main loop


////// External Interrupt function with software switch debounce:
void IRAM_ATTR handleInterrupt(){
  
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  
  // If interrupts come faster than 350ms, assume it's a bounce and ignore:
  if (interrupt_time - last_interrupt_time > 200){
    // Determine which switch was triggered, and which track to play:
      if(!digitalRead(GAME_OVER_PIN)){ 
        loadTrack = GAME_OVER;
        event = GAME_OVER;
        score = 0; 
      }
      
      else if(!digitalRead(TARGET_PIN)){ 
        loadTrack = TARGET;
        event = TARGET;
        score = score + 5;
      }

      else if(!digitalRead(SLINGSHOT_PIN_1)){ 
        loadTrack = SLINGSHOT;
        event = SLINGSHOT;
        score = score + 5;
      }

      else if(!digitalRead(SLINGSHOT_PIN_2)){ 
        loadTrack = SLINGSHOT;
        event = SLINGSHOT;
        score = score + 5;
      }
      
      playing = 1;
  }
  
  last_interrupt_time = interrupt_time;
}

void setup() {

////// Pin Setup:
// Configure trigger pins to inputs with internal pull-up resistors enabled:
  pinMode(GAME_OVER_PIN,INPUT_PULLUP);
  pinMode(TARGET_PIN,INPUT_PULLUP);
  pinMode(SLINGSHOT_PIN_1,INPUT_PULLUP);
  pinMode(SLINGSHOT_PIN_2,INPUT_PULLUP);
  
// Create interrupts for each trigger:
  attachInterrupt(digitalPinToInterrupt(GAME_OVER_PIN),handleInterrupt,FALLING);
  attachInterrupt(digitalPinToInterrupt(TARGET_PIN),handleInterrupt,FALLING);
  attachInterrupt(digitalPinToInterrupt(SLINGSHOT_PIN_1),handleInterrupt,FALLING);
  attachInterrupt(digitalPinToInterrupt(SLINGSHOT_PIN_2),handleInterrupt,FALLING);
  
////// Audio Setup:
  out = new AudioOutputI2S();
  aac = new AudioGeneratorAAC();
  out -> SetGain(0.25);           // Set the volume
  
  xTaskCreatePinnedToCore(
                    AudioTask,    // Task
                    "AudioTask",  // Task name string
                    10000,        
                    NULL,         
                    1,            // Priority
                    NULL,         
                    0);           // Use core 0
  delay(500);

////// LED Setup:
  pinMode(BLUE, OUTPUT);          
  pinMode(RED, OUTPUT);           
  pinMode(GREEN, OUTPUT);         

  xTaskCreatePinnedToCore(
                    LEDTask,      
                    "LEDTask",    
                    10000,        
                    NULL,         
                    1,            
                    NULL,      
                    1);           // Use core 1                    
  delay(500); 
  

////// 7-segment Setup:
  #ifndef __AVR_ATtiny85__
  #endif
// Score Display:
  I2CMatrix_score.begin(33,32);  // V_IO is 5 volts, SDA, SCL
  matrix_score.begin(0x70, &I2CMatrix_score);
  matrix_score.clear();
  matrix_score.print(0, DEC);
  matrix_score.writeDisplay();
  
// Game Number Display: 
  I2CMatrix_game.begin(23,21);   // V_IO is 5 volts, SDA, SCL
  matrix_game.begin(0x70, &I2CMatrix_game);
  matrix_game.clear();
  matrix_game.print("NDEE");
  matrix_game.writeDisplay();

}

////// LED Task:
void LEDTask( void * pvParameters ){
    
  for(;;){
    led_ambient();
    TIMERG0.wdt_wprotect=TIMG_WDT_WKEY_VALUE;    // shutoff watchdog timer in core
    TIMERG0.wdt_feed=1;
    TIMERG0.wdt_wprotect=0;
  }
}


//// Audio Task:
void AudioTask( void * pvParameters ){
  
  for(;;){
    // Stop the current track if playing
    if(playing && aac->isRunning()) aac->stop();

    // Choose event based on what was triggered:
    if(loadTrack == GAME_OVER){
      file_gameOver = new AudioFileSourcePROGMEM(gameOver, sizeof(gameOver));
      aac -> begin(file_gameOver,out); // Start playing the track loaded
     
      playing = 0;
      while(playing == 0){
        if (aac->isRunning()) {
          aac->loop();
        } 
        else{
          playing = 1;
          loadTrack = 0;  // don't play any sounds twice
        }
      }
           
    }
    
    else if(loadTrack == TARGET){
      file_target = new AudioFileSourcePROGMEM(target, sizeof(target));
      aac -> begin(file_target,out); // Start playing the boost sound   
      playing = 0;
      while(playing == 0){
        if (aac->isRunning()) {
          aac->loop();
        } 
        else{
          playing = 1;
          loadTrack = 0;  // don't play any sounds twice
        }
      }

    }

    else if(loadTrack == SLINGSHOT){
      file_slingshot = new AudioFileSourcePROGMEM(slingshot, sizeof(slingshot));
      aac -> begin(file_slingshot,out); //Start playing the track loaded
      playing = 0;
      while(playing == 0){
        if (aac->isRunning()) {
          aac->loop();
        } 
        else{
          playing = 1;
          loadTrack = 0;  // don't play any sounds twice
        }
      }
      
    }
    
    else{
           
    } //end of ELSE statement
  } 
}


////// Main Loop:
void loop() {
// Interrupt has triggered an event:
  if(event){ 

    // Choose event based on what was triggered:
    if(event == GAME_OVER){  
      matrix_score.print(0, DEC);
      matrix_score.writeDisplay();
      led_game_over();
    } 
    
    else if(event == TARGET){
      matrix_score.print(score, DEC);
      matrix_score.writeDisplay();      
      led_score();
    }

    else if(event == SLINGSHOT){
      matrix_score.print(score, DEC);
      matrix_score.writeDisplay();   
      led_score();
    }
    
    event = 0;              // Event is over: reset
  }

// Nothing is currently happening:
  else{
  }

}

////// LED Color Functions:
void rgb_off(){
  digitalWrite(RED, LOW);
  digitalWrite(BLUE, LOW);
  digitalWrite(GREEN, LOW);
}

void rgb_red(){
  digitalWrite(RED, HIGH);
  digitalWrite(BLUE, LOW);
  digitalWrite(GREEN, LOW);
}

void rgb_blue(){
  digitalWrite(RED, LOW);
  digitalWrite(BLUE, HIGH);
  digitalWrite(GREEN, LOW);
}

void rgb_green(){
  digitalWrite(RED, LOW);
  digitalWrite(BLUE, LOW);
  digitalWrite(GREEN, HIGH);
}

void rgb_magenta(){
  digitalWrite(RED, HIGH);
  digitalWrite(BLUE, HIGH);
  digitalWrite(GREEN, LOW);
}

void rgb_yellow(){
  digitalWrite(RED, HIGH);
  digitalWrite(BLUE, LOW);
  digitalWrite(GREEN, HIGH);
}

void rgb_cyan(){
  digitalWrite(RED, LOW);
  digitalWrite(BLUE, HIGH);
  digitalWrite(GREEN, HIGH);
}

void led_ambient(){
  rgb_green();
  delay(430);
  rgb_blue();
  delay(430);
  rgb_yellow();
  delay(430);
}

void led_score(){
  rgb_red();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_red();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_red();
  delay(250);
  rgb_blue();
  delay(250);
}

void led_game_over(){
  rgb_red();
  delay(250);
  rgb_red();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_magenta();
  delay(250);
  rgb_magenta();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_red();
  delay(250);
  rgb_red();
  delay(250);
  rgb_blue();
  delay(250);
  rgb_magenta();
  delay(250);
  rgb_off();
}
